/**
 * File: can-spell.cpp
 * -------------------
 * can-spell helps the user decide whether or not a string
 * can be spelled out using just the symbols in the periodic
 * table.
 */

#include <string>    // for string class
#include <iostream>  // for cout, endl
#include <cctype>    // for toupper
#include <algorithm> // for min
using namespace std;

#include "console.h"
#include "set.h"     // for Set class
#include "simpio.h"  // for getLine
#include "strlib.h"  // for strlib

/**
 * Function: addPeriodicTableElements
 * ----------------------------------
 * Accepts the presumably empty set and adds all of the
 * elements in the periodic table.  Note the beautiful use
 * of += and , to compactly add the entire set of elements.
 */
static void addPeriodicTableElements(Set<string>& symbols) {
    symbols += "H", "He";
    symbols += "Li", "Be", "B", "C", "N", "O", "F", "Ne";
    symbols += "Na", "Mg", "Al", "Si", "P", "S", "Cl", "Ar";
    symbols += "K", "Ca", "Sc", "Ti", "V", "Cr", "Mn", "Fe",
               "Co", "Ni", "Cu", "Zn", "Ga", "Ge", "As", "Se", "Br", "Kr";
    symbols += "Rb", "Sr", "Y", "Zr", "Nb", "Mo", "Tc", "Ru", "Rh", "Pd", "Ag",
               "Cd", "In", "Sn", "Sb", "Te", "I", "Xe";
    symbols += "Cs", "Ba", "La", "Ce", "Pr", "Nd", "Pm", "Sm", "Eu", "Gd", "Tb",
               "Dy", "Ho", "Er", "Tm", "Yb", "Lu", "Hf", "Ta", "W", "Re", "Os",
               "Ir", "Pt", "Au", "Hg", "Tl", "Pb", "Bi", "Po", "At", "Rn";
    symbols += "Fr", "Ra", "Ac", "Th", "Pa", "U", "Np", "Pu", "Am", "Cm", "Bk",
               "Cf", "Es", "Fm", "Md", "No", "Lr", "Rf", "Db", "Sg", "Bh",
               "Hs", "Mt", "Ds", "Rg", "Cn", "Uut", "Fl", "Uup", "Lv", "Uus", "Uuo";
}

/**
 * Function: elementize
 * --------------------
 * Utility function to take any string and normalize it
 * to appear to be an element in the periodic table.
 */
static string elementize(const string& str) {
    if (str.empty()) return str;
    string copy = toLowerCase(str);
    copy[0] = toupper(copy[0]);
    return copy;
}

/**
 * Function: canSpell
 * ------------------
 * Relies on recursive backtracking to decide whether or not
 * the provided word can be spelled using just elements of the
 * periodic table.
 */
static const size_t kMaxSymbolLength = 3;
/* static */ bool canSpell(const string& word, const Set<string>& symbols) {
	if (word.empty()) return true;
	
    size_t length = word.size();
    for (size_t i = 1; i <= min(kMaxSymbolLength, length); i++) {
        string symbol = elementize(word.substr(0, i));
		if (symbols.contains(symbol) &&
            canSpell(word.substr(i), symbols)) {
			return true;
		}
	}
    
	return false;
}

/**
 * Function: canSpell
 * ------------------
 * Provides the same functionality as the first version with the side
 * effect that the official spelling--with correct capitalization, as with,
 * say, "HOOLiGaNISm"--be laid down in the space provided by footprint.
 */
static bool canSpell(const string& word,
                     const Set<string>& symbols, Stack<string>& footprint) {
    if (word.empty()) return true;

    size_t length = word.size();
    for (size_t i = 1; i <= min(kMaxSymbolLength, length); i++) {
        string symbol = elementize(word.substr(0, i));
		if (symbols.contains(symbol) &&
            canSpell(word.substr(i), symbols, footprint)) {
            footprint.push(symbol);
			return true;
		}
	}
    
	return false;
}

/**
 * Function: main
 * --------------
 * Provides the entry point for the entire program, which
 * repeatedly prompts the user for words, and then informs
 * the user whether that word can be spelled with a second
 * alphabet: where the letters are the spellings of the elements
 * in the periodic table (e.g. "O", "Li", "Sm", etc.)
 */
int main() {
    Set<string> symbols;
    addPeriodicTableElements(symbols);
	while (true) {
		string word = getLine("Enter a word: ");
		if (word.empty()) break;
        Stack<string> footprint;
		if (canSpell(word, symbols, footprint)) {
            cout << "That can be spelled as \"";
            while (!footprint.isEmpty()) {
                cout << footprint.pop();
            }
            cout << "\"." << endl;
        } else {
            cout << "That's just not possible." << endl;
        }
	}

    return 0;
}
