/**
 * File: cubic-decompositions.cpp
 * ------------------------------
 * helps the user decide whether the cube of an integer
 * can be written as the sum of two or more smaller cubes.
 */

#include <string>    // for string class
#include <iostream>  // for cout, endl
#include <cctype>    // for toupper
#include <algorithm> // for min
using namespace std;

#include "console.h"
#include "set.h"     // for Set class
#include "simpio.h"  // for getInteger

/**
 * Function: cubicDecompositionExists
 * ----------------------------------
 * Returns true if and only if the value provided via remaining can
 * be written as a sum of distinct cubes with bases less or equal to
 * maxRoot, limited to the number of terms provided via the last argument.
 *
 * If a cubic decomposition exists, all of the roots are dropped into
 * the cubicRoots set.  If a cubic decomposition doesn't exist, the
 * set is left alone.
 */
static bool cubicDecompositionExists(int remaining, int maxRoot,
                                     Set<int>& cubicRoots, int maxTerms) {
	if (remaining == 0) return true;
    if (remaining < 0) return false;
    if (maxTerms == 0) return false;
    
    for (int s = maxRoot; s > 0; s--) {
        if (cubicDecompositionExists(remaining - s * s * s, s - 1,
                                     cubicRoots, maxTerms - 1)) {
 			cubicRoots += s;
  			return true;
        }
	}
    
	return false;
}

/**
 * Function: cubicDecompositionExists
 * ----------------------------------
 * Returns true if and only if n^3 can be written as a sum
 * of two or more distinct cubes.  If not, the provided set is
 * left alone.  If so, the set is populated with the bases of
 * the cubes.
 */
static bool cubicDecompositionExists(int n, Set<int>& cubicRoots) {
	for (int maxTerms = 2; maxTerms <= n; maxTerms++) {
        if (cubicDecompositionExists(n * n * n, n - 1, cubicRoots, maxTerms))
			return true;
 	}
    
	return false;
}

/**
 * Function: main
 * --------------
 * Provides the entry point into the entire program, which tests
 * the second cubicDecompositionExists function written above.
 */
int main() {
	while (true) {
        int n = getInteger("Enter a number [nonpositive to quit]: ");
        if (n <= 0) break;
        Set<int> cubicRoots;
        if (cubicDecompositionExists(n, cubicRoots)) {
            cout << "We found a cubic decomposition for " << n << "^3." << endl;
            cout << "You see, " << n << "^3 just happens to equal ";
            for (int root: cubicRoots) {
                if (root != cubicRoots.first()) cout << " + ";
                cout << root << "^3";
            }
            cout << ".";
        } else {
            cout << "No cubic decomposition exists.";
        }
        cout << endl << endl;
	}

    return 0;
}
